package com.muyu.vehicle.core;

import com.muyu.system.handle.SystemHandler;
import com.muyu.vehicle.VehicleInstance;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author DongZl
 * @description: 车辆容器
 * @Date 2023-11-16 下午 02:30
 */
public class LocalContainer {

    private static final Map<String, Map<String, VehicleInstance>> tenantVehicleDataMap
            = new ConcurrentHashMap<>();

    /**
     * 通过租户ID获取租户下车辆
     * @param tenantId 租户ID
     * @return 车辆内容
     */
    public static Map<String, VehicleInstance> getVehicleDataMap(String tenantId) {
        return tenantVehicleDataMap
                .computeIfAbsent(tenantId, k -> new ConcurrentHashMap<>());
    }

    /**
     * 默认获取档期啊租户下车辆
     * @return 车辆内容
     */
    public static Map<String, VehicleInstance> getTenantVehicleDataMap() {
        return getVehicleDataMap(SystemHandler.getTenantId());
    }

    /**
     * 添加车辆
     * @param vehicleInstance 车辆信息
     */
    public static void setVehicleInstance(List<VehicleInstance> vehicleInstance){
        vehicleInstance.forEach(LocalContainer::setVehicleInstance);
    }
    public static void setVehicleInstance(VehicleInstance vehicleInstance){
        String vin = vehicleInstance.getVehicleInfo().getVin();
        Map<String, VehicleInstance> vehicleDataMap = getVehicleDataMap(vehicleInstance.getTenantId());
        if (!vehicleDataMap.containsKey(vin)) {
            vehicleDataMap.put(vin, vehicleInstance);
        }
    }

    /**
     * 获取车辆信息
     * @param vin vin
     * @return 车辆实例
     */
    public static VehicleInstance getVehicleInstance(String vin){
        Map<String, VehicleInstance> vehicleDataMap = getTenantVehicleDataMap();
        return vehicleDataMap.get(vin);
    }

    /**
     * 车辆上线总数
     * @return
     */
    public static long total () {
        Map<String, VehicleInstance> vehicleDataMap = getTenantVehicleDataMap();
        return vehicleDataMap.size();
    }

    public static Collection<VehicleInstance> getVehicleInstanceAll () {
        Map<String, VehicleInstance> vehicleDataMap = getTenantVehicleDataMap();
        return vehicleDataMap.values();
    }


    /**
     * 获取在线车辆
     * @return 在线车辆集合
     */
    public static List<VehicleInstance> getOnlineVehicleInstance(){
        Map<String, VehicleInstance> vehicleDataMap = getTenantVehicleDataMap();
        return vehicleDataMap.values().stream().filter(VehicleInstance::isOnline).toList();
    }
    /**
     * 获取在线车辆
     * @return 在线车辆VIN集合
     */
    public static List<String> getOnlineVehicleVin(){
        return getOnlineVehicleInstance()
                .stream()
                .map(VehicleInstance::getVin)
                .toList();
    }

    /**
     * 获取离线车辆
     * @return 离线车辆集合
     */
    public static List<VehicleInstance> getOfflineVehicleInstance(){
        Map<String, VehicleInstance> vehicleDataMap = getTenantVehicleDataMap();
        return vehicleDataMap.values().stream().filter(vehicleInstance -> !vehicleInstance.isOnline()).toList();
    }

    /**
     * 通过VIN删除
     * @param vin 车辆VIN
     */
    public static void removeByVin(String vin) {
        Map<String, VehicleInstance> vehicleDataMap = getTenantVehicleDataMap();
        vehicleDataMap.remove(vin);
    }
}
